#!/usr/local/bin/python
#
# reviewboard-post-commit-hook
# This script should be invoked from the subversion post-commit hook like this:
#
# REPOS="$1"
# REV="$2"
# /usr/bin/python /some/path/reviewboard-post-commit-hook.py "$REPOS" "$REV" || exit 1
#
# Searches the commit message for text in the form of:
#   publish review - publishes a review request
#   draft review - creates a draft review request
#
# The space before 'review' may be ommitted.
#
# The log message is interpreted for review request parameters:
#    summary = first 150 chars 
#    description = entire log message
#    existing review updated if log message includes 'review:[0-9]+'
#    bugs added to review if log message includes commands as defined in
#      supported_ticket_cmds
#
# The review request is created out of a diff between the current revision (M)
# and either the previous version (M-1) or any other revision if the log
# message includes 'prevrev:[0-9]+'.  The latter option is useful for creating
# a review that spans multiple commits.
#
# An example commit message is:
#
#    Changed blah and foo to do this or that.  Publish review ticket:1 review:2 prevrev:3.
#
# This would update the existing review 2 with a diff between this commit and
# revision 3.  It would place the entire log message in the review summary and
# description, and put bug id 1 in the bugs field.
#

#
# User configurable variables
#

# path to post-review script, which must be renamed/copied to
# postreview.py so that it can be loaded as a module.
POSTREVIEW_PATH = '/home/svn/repositories/devel/hooks/postreview.py/'
SVNLOOK_PATH = '/usr/local/bin/svnlook'
REPOSITORY_URL = 'https://***************/svn/devel'
REVIEWBOARD_SERVER = 'http://127.0.0.1:443/reviewboard/'
ADMIN_USERNAME = 'admin'
ADMIN_PASSWORD = '### ENTER PASSWORD ###'

#
# end user configurable variables
#

import sys
import re
import subprocess

# import the Review Board 'postreview' script as a module
sys.path.append(POSTREVIEW_PATH)
import postreview

# list of trac commands from trac-post-commit-hook.py.
# numbers following these commands will be added to the bugs
# field of the review request.
supported_ticket_cmds = {'close':      '_cmdClose',
                         'closed':     '_cmdClose',
                         'closes':     '_cmdClose',
                         'fix':        '_cmdClose',
                         'fixed':      '_cmdClose',
                         'fixes':      '_cmdClose',
                         'addresses':  '_cmdRefs',
                         're':         '_cmdRefs',
                         'references': '_cmdRefs',
                         'refs':       '_cmdRefs',
                         'see':        '_cmdRefs'}

ticket_prefix = '(?:#|(?:ticket|issue|bug)[: ]?)'
ticket_reference = ticket_prefix + '[0-9]+'
ticket_command = (r'(?P<action>[A-Za-z]*).?'
                  '(?P<ticket>%s(?:(?:[, &]*|[ ]?and[ ]?)%s)*)' %
                  (ticket_reference, ticket_reference))

def main():
    if len(sys.argv) != 3:
        print >> sys.stderr, 'Usage: %s <repos> <txn>' % sys.argv[0]
        sys.exit(1)

    repos = sys.argv[1]
    txn = sys.argv[2]

    # get the log message
    svnlook = [SVNLOOK_PATH, 'log', '-t', txn, repos]
    log = subprocess.Popen(svnlook, stdout=subprocess.PIPE).communicate()[0]

    # error if log message is blank
    if len(log.strip()) < 1:
        sys.stderr.write("Log message is empty, no review request created")
        sys.exit(1)

    # get the author
    svnlook = [SVNLOOK_PATH, 'author', '-t', txn, repos]
    author = subprocess.Popen(svnlook, stdout=subprocess.PIPE).communicate()[0]
    author = author.split('\n')[0]
    author = author.split('@')[0].replace('.','')

    # error if author is blank
    if len(author.strip()) < 1:
        sys.stderr.write("Author is blank, no review request created")
        sys.exit(1)

    # get the diff
    svnlook = [SVNLOOK_PATH, 'diff', '-t', txn, repos]
    diff = subprocess.Popen(svnlook, stdout=subprocess.PIPE).communicate()[0]
    filename = '/tmp/reviewboard-diff-%s' % (txn)
    try:
        fd = file(filename, 'wb')
        fd.write(diff)
        fd.close()
    except:
        sys.stderr.write("Unable to write temporary diff, abort.")
        sys.exit(1)

    # check whether to create a review, based on presence of word 
    # 'review' with prefix
    review=r'(?:publish|draft)(?: )?review'
    if None == re.search(review, log, re.M | re.I):
        print 'No review requested'
        sys.exit(0)

    # check for update to existing review
    m = re.search(r'review:([0-9]+)', log, re.M | re.I)
    if m:
        reviewid='--review-request-id=' + m.group(1)
    else:
        reviewid=''

    # check whether to publish or leave review as draft
    if re.search(r'draft(?: )?review', log, re.M | re.I):
        publish=''
    else:
        publish='-p'

    # get bug numbers referenced in this log message
    ticket_command_re = re.compile(ticket_command)
    ticket_re = re.compile(ticket_prefix + '([0-9]+)')

    ticket_ids = []
    ticket_cmd_groups = ticket_command_re.findall(log)
    for cmd, tkts in ticket_cmd_groups:
        funcname = supported_ticket_cmds.get(cmd.lower(), '')
        if funcname:
            for tkt_id in ticket_re.findall(tkts):
                ticket_ids.append(tkt_id)

    if ticket_ids:
        bugs='--bugs-closed=' + ','.join(ticket_ids)
    else:
        bugs=''

    # other parameters for postreview
    summary='--summary=' + log[:150].splitlines().pop(0)
    description='--description=' + log
    submitas='--submit-as=' + author
    difffile='--diff=' + filename
    repo='--repository='+REPOSITORY_URL
    rbserver='--server='+REVIEWBOARD_SERVER
    username='--username='+ADMIN_USERNAME
    password='--password='+ADMIN_PASSWORD
    args = ['post-review', '-d', publish, summary, description, submitas, difffile,
            reviewid, bugs, repo, rbserver, username, password]

    # Run Review Board post-review script
    print '==> Post review for %s, transaction %s' % (author, txn)
    ret = postreview.main(args)
    try:
        os.unlink(filename)
    except:
        pass
    return ret

if __name__ == '__main__':
    sys.exit(main())
